#include <CP210x.h>
#include <usbhub.h>
#include "pgmstrings.h"
#include <SPI.h>
#include "DFRobot_SpeechSynthesis.h"

DFRobot_SpeechSynthesis_UART ss;
// 调试开关
#define NCDEBUG 0
#define BUTTON_PIN 25

// 每次收到的有效数据长度
#define TOTAL 15
long int LastRead = 0;

class CP210xAsync : public CP210xAsyncOper
{
  public:
    uint8_t OnInit(CP210x *pftdi);
};

// CP210X 初始化
uint8_t CP210xAsync::OnInit(CP210x *cp210x)
{
  uint8_t rcode = 0;
  rcode = cp210x->IFCEnable();

  if (rcode)
  {
    return rcode;
  }

  // Victor 通讯波特率位 2400
  rcode = cp210x->SetBaudRate(2400);

  if (rcode)
  {
    return rcode;
  }

  // 8 Bits
  rcode = cp210x->SetDataBits(8);
  if (rcode)
  {
    return rcode;
  }

  // 1 停止位
  rcode = cp210x->SetStopBits(CP210X_STOP_BITS_1);
  if (rcode)
  {
    return rcode;
  }

  // 无奇偶校验
  rcode = cp210x->SetParity(CP210X_PARITY_NONE);
  if (rcode)
  {
    return rcode;
  }

  // 无流控
  rcode = cp210x->SetFlowControl(CP210X_FLOW_CONTROL_OFF);
  if (rcode)
  {
    return rcode;
  }

  return rcode;
}

USB Usb;

CP210xAsync cp210xAsync;
CP210x cp210x(&Usb, &cp210xAsync);

boolean Pressed = false;

void Button_Pressed() {
  if (millis()-LastRead > 3000) {
      Pressed = true;
      LastRead=millis();
    }
}
void setup()
{
  Serial.begin( 115200 );
  Serial.println("Start");

  Serial2.begin(115200);
  //Init speech synthesis sensor
  ss.begin(Serial2);

  if (Usb.Init() == -1)
    Serial.println("OSC did not start.");

  pinMode(BUTTON_PIN, INPUT_PULLUP);//将中断的引脚设置为输入PULLUP模式
  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN), Button_Pressed, RISING); //设置触发中断的模式和中断服务函数

  delay( 200 );
}

// 将两个表示数值的字节拼接成字符串，first 用来表示是否为第一位数值
String ByteToString(byte b1, byte b2, boolean first) {
  // 取第一个和第二个字节的Bit[3:0]组成一个数值，同时去掉Bit7
  byte tmp = ((b1 << 4) + (b2 & 0xF)) & 0x7F;
  //                       0     1     2     3     4     5     6     7     8     9   L
  byte HexToValue[] = { 0x7D, 0x05, 0x5B, 0x1F, 0x27, 0x3E, 0x7E, 0x15, 0x7F, 0x3F , 0x68};
  String Result;
  if (first) {
    // 第一个字节的 Bit3 表示符号
    if ((b1 & 0x8) != 0) {
      Result = Result + '-';
    }
  } else {
    // 其余字节的 Bit3 表示前面的小数点
    if ((b1 & 0x8) != 0)
    {
      Result = Result + '.';
    }
  }

  // 查表来确定当前的数值
  for (int i = 0; i < 11; i++) {
    if (tmp == HexToValue[i]) {
      if (i == 10)
      {
        // 无效数值使用 0L 表示
        Result = Result + "L";
      }
      else {
        Result = Result + i;
      }
      break;
    }
  }
  return Result;
}

// 将获得的数组转化为字符串
String ParserResult(byte *readBuffer)
{
  String Value = "";
  /* 前缀有 AC DC 分别表示交流和直流
     本代码不做处理。同样还有 AUTO 和 RS232
    if ((readBuffer[0] & 8) != 0) {
    Value = Value + "AC";
    }
    if ((readBuffer[0] & 4) != 0)
    {
    Value = Value + "DC";
    }
    if ((readBuffer[0] & 2) != 0)
    {
    Value = Value + "AUTO";
    }
    if ((readBuffer[0] & 1) != 0)
    {
    Value = Value + "RS232";
    }
  */
  // 将收到的数据转化为数值
  Value = Value + ByteToString(readBuffer[1], readBuffer[2], true);
  Value = Value + ByteToString(readBuffer[3], readBuffer[4], false);
  Value = Value + ByteToString(readBuffer[5], readBuffer[6], false);
  Value = Value + ByteToString(readBuffer[7], readBuffer[8], false);

  // 接下来处理数据的单位
  if ((readBuffer[9] & 8) != 0) {
    Value += "u";
  }
  if ((readBuffer[9] & 4) != 0)
    Value += "n";
  if ((readBuffer[9] & 2) != 0)
    Value += "K";

  if ((readBuffer[10] & 8) != 0)
  {
    Value += "m";
  }
  if ((readBuffer[10] & 4) != 0)
  {
    Value += "%";
  }
  if ((readBuffer[10] & 2) != 0)
  {
    Value += "M";
  }

  if ((readBuffer[11] & 8) != 0)
  {
    Value += "F";
  }
  if ((readBuffer[11] & 4) != 0)
  {
    Value += "O";
  }
  if ((readBuffer[12] & 8) != 0)
  {
    Value += "A";
  }
  if ((readBuffer[12] & 4) != 0)
  {
    Value += "V";
  }
  if ((readBuffer[12] & 2) != 0)
  {
    Value += "Hz"; // Hz
  }
  if ((readBuffer[13] & 4) != 0)
  {
    Value += "mV"; //  mV
  }
  if ((readBuffer[13] & 2) != 0)
  {
    Value += "C";
  }
  return Value;
}

// 将字符串转化为数值，比如：123 是“一百二十三”
String ValueToValue(String Str) {
  String Result;
  String DataBuffer[13] = {{"零"}, {"一"}, {"二"}, {"三"}, {"四"}, {"五"}, {"六"}, {"七"}, {"八"}, {"九"}, {"十"}, {"百"}, {"千"}};
  int Value = Str.toInt();
  // 千位不为零
  if (Value / 1000 != 0)
  {
    // 读出千位
    Result += DataBuffer[Value / 1000];
    // 读出"千"
    Result += DataBuffer[12];
    // 如果百位为 0 ,那么直接读出"零"
    if (Value / 100 % 100 == 0)
    {
      Result += DataBuffer[0];
    }
  }

  // 百位不为零
  if (Value / 100 != 0)
  {
    // 读出百位
    Result += DataBuffer[Value / 100];
    // 读出"百"
    Result += DataBuffer[11];
    // 如果十位为 0 ,那么直接读出"零"
    if (Value / 10 % 10 == 0)
    {
      Result += DataBuffer[0];
    }
  }
  else
  {
    // 十位不为零
    if ((Value / 10 % 10) != 0)
    {
      // 读出十位（不含0的情况）
      if (Value / 10 == 1) { // 13 这种直接读“十三”
        Result += DataBuffer[10];
      } else {  // 23 这种读出“二十三”
        Result += DataBuffer[Value / 10 % 10];
        Result += DataBuffer[10];
      }
    }
  }

  // 个位不为零
  if ((Value % 10) != 0)
  {
    Result += DataBuffer[Value % 10];
  }

  // 零特别处理
  if (Value == 0)
  {
    Result += DataBuffer[0];
  }
  return Result;
}

// 将字符串转化为数值，比如：123 是“一二三”
String ValueToNumber(String Str) {
  String Result;
  String DataBuffer[11] = {{"零"}, {"一"}, {"二"}, {"三"}, {"四"}, {"五"}, {"六"}, {"七"}, {"八"}, {"九"}};
  int Value = Str.toInt();

  while (Str != "") {
    Result += DataBuffer[Str.charAt(0) - 0x30];
    Str.remove(0, 1);
  }
  return Result;
}
String ResultToWord(String V) {
  String Result = "";
  // 处理负号
  if (V.charAt(0) == '-') {
    V.remove(0, 1);
    Result += "负";
  }
  // 特别处理结果为 0 的情况
  if ((V.indexOf("0.000") != -1) ||
      (V.indexOf("00.00") != -1) ||
      (V.indexOf("000.0") != -1)) {
    V.remove(0, 5);
    Result += "零";
  } else {
    // 去掉数值中最前的 0
    while ((V.charAt(0) == '0') && (V.charAt(0) == '0')) {
      V.remove(0, 1);
    }

    String Num = "";
    // 处理小数点之前的数值
    while ((V.charAt(0) >= '0') && (V.charAt(0) <= '9')) {
      Num += V.charAt(0);
      V.remove(0, 1);
    }
    // 比如这里 Num=123 需要读成 一百二十三
    Result += ValueToValue(Num);
    // 处理小数点
    if ((V.charAt(0) == '.')) {
      Result += "点";
      V.remove(0, 1);
    }

    Num = "";
    // 处理小数点之后的数值
    while ((V.charAt(0) >= '0') && (V.charAt(0) <= '9')) {
      Num += V.charAt(0);
      V.remove(0, 1);
    }
    // 比如，这里是 123 需要读成 “一二三”
    Result += ValueToNumber(Num);

  }
  // 处理单位
  if (V.indexOf("mV") != -1) {
    Result += "毫伏";
  } else if (V.indexOf("V") != -1) {
    Result += "伏";
  }
  if (V.indexOf("uA") != -1) {
    Result += "微安";
  } else if (V.indexOf("mA") != -1) {
    Result += "毫安";
  } else if (V.indexOf("A") != -1) {
    Result += "安";
  }
  if (V.indexOf("KO") != -1) {
    Result += "千欧";
  } else if (V.indexOf("MO") != -1) {
    Result += "兆欧";
  } else if (V.indexOf("O") != -1) {
    Result += "欧";
  }
  if (V.indexOf("nF") != -1) {
    Result += "纳法";
  } else if (V.indexOf("uF") != -1) {
    Result += "微法";
  }
  if (V.indexOf("KHz") != -1) {
    Result += "千赫兹";
  } else if (V.indexOf("MHz") != -1) {
    Result += "兆赫兹";
  } else if (V.indexOf("Hz") != -1) {
    Result += "赫兹";
  }
  if (V.indexOf("C") != -1) {
    Result += "摄氏度";
  }

  return Result;
}

// 存放拼接出来的有效数据包
uint8_t  Data[TOTAL];
uint8_t  DataPtr = 0;
String   Final;


void loop()
{
  Usb.Task();

  if ( Usb.getUsbTaskState() == USB_STATE_RUNNING )
  {
    uint8_t  rcode;
    uint8_t  Buffer[TOTAL];
    uint8_t  n = TOTAL - DataPtr;

    // 这里要求一次取 TOTAL字节，但是通常是不到 TOTAL字节
    // 所以需要每次取出之后拼接在一起
    rcode = cp210x.RcvData((uint16_t *)&n, (uint8_t *)&Buffer[0]);
    if (rcode && rcode != hrNAK)
      ErrorMessage<uint8_t>(PSTR("Ret"), rcode);

    if ((n != 0) && (Pressed != true)) {
      // 这里可以输出收到的数据包
      if (NCDEBUG) {
        for (byte i = 0; i < n; i++) {
          Serial.print(Buffer[i], HEX);
          Serial.print(" ");
        }
        Serial.println("");
      }

      // 检查收到的数据Bit[7:4] 是否有效
      if ((Buffer[0] >> 4) != DataPtr + 1) {
        DataPtr = 0;
        return;
      };

      // 将收到的数据拼接到 Data 中
      for (byte i = 0; i < n; i++) {
        Data[DataPtr + i] = Buffer[i];
      }

      DataPtr += n;

      // 如果收到了 TOTAL-1 个字符，就说明收到足够的数据
      if (DataPtr == TOTAL - 1) {
        // 这里可以输出一下收到的数据包
        if (NCDEBUG) {
          for (byte i = 0; i < DataPtr; i++) {
            Serial.print(Data[i], HEX);
            Serial.print(" ");
          }
          Serial.println(" ");
        }

        DataPtr = 0;
        // 将收到的数据包解析为字符串
        Final = ResultToWord(ParserResult(Data));
        if (NCDEBUG) {
          Serial.println(Final);
        }
        Serial.println(Final);
      }
    }
  }
  if (Pressed) {
/*    long int elsp = millis();
    char Buf[4];
    Buf[0] = Final.charAt(ReadOut * 3);
    Buf[1] = Final.charAt(ReadOut * 3 + 1);
    Buf[2] = Final.charAt(ReadOut * 3 + 2);
    Buf[3] = 0;
    ss.speak(Buf);
    Serial.println(Buf);
    Serial.println(millis() - elsp);
    if (ReadOut == (Final.length() / 3)) {
      Pressed = false;
      ReadOut = 0;
    } else {
      ReadOut++;
    }
    */
    ss.speak(Final);
    Pressed = false;
    
  }
}
